=== added directory 'plugin/memcached_stats'
=== added file 'plugin/memcached_stats/memcached_stats.cc'
--- plugin/memcached_stats/memcached_stats.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_stats/memcached_stats.cc	2009-09-29 19:34:58 +0000
@@ -0,0 +1,201 @@
+/*
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/show.h"
+#include "drizzled/gettext.h"
+#include "drizzled/info_schema.h"
+
+#include "stats_table.h"
+
+#include <string>
+#include <map>
+
+using namespace std;
+using namespace drizzled;
+
+/*
+ * Vectors of columns for I_S tables.
+ */
+static vector<const ColumnInfo *> memcached_stats_columns;
+
+/*
+ * Methods for I_S tables.
+ */
+static InfoSchemaMethods *memcached_stats_methods= NULL;
+
+/*
+ * I_S tables.
+ */
+static InfoSchemaTable *memcached_stats_table= NULL;
+
+/*
+ * System variable related variables.
+ */
+static char *sysvar_memcached_servers= NULL;
+static const char DEFAULT_SERVERS_STRING[]= "localhost:11211";
+
+/**
+ * Populate the vectors of columns for each I_S table.
+ *
+ * @return false on success; true on failure.
+ */
+static bool initColumns()
+{
+  if (createMemcachedStatsColumns(memcached_stats_columns))
+  {
+    return true;
+  }
+
+  return false;
+}
+
+/**
+ * Clear the vectors of columns for each I_S table.
+ */
+static void cleanupColumns()
+{
+  clearMemcachedColumns(memcached_stats_columns);
+}
+
+/**
+ * Initialize the methods for each I_S table.
+ *
+ * @return false on success; true on failure
+ */
+static bool initMethods()
+{
+  memcached_stats_methods= new(std::nothrow) 
+    MemcachedStatsISMethods(sysvar_memcached_servers);
+  if (! memcached_stats_methods)
+  {
+    return true;
+  }
+
+  return false;
+}
+
+/**
+ * Delete memory allocated for the I_S table methods.
+ */
+static void cleanupMethods()
+{
+  delete memcached_stats_methods;
+}
+
+/**
+ * Initialize the I_S tables related to memcached.
+ *
+ * @return false on success; true on failure
+ */
+static bool initMemcachedTables()
+{
+  memcached_stats_table= new(std::nothrow) InfoSchemaTable("MEMCACHED_STATS",
+                                                           memcached_stats_columns,
+                                                           -1, -1, false, false, 0,
+                                                           memcached_stats_methods);
+  if (! memcached_stats_table)
+  {
+    return true;
+  }
+
+  return false;
+}
+
+/**
+ * Delete memory allocated for the I_S tables.
+ */
+static void cleanupMemcachedTables()
+{
+  delete memcached_stats_table;
+}
+
+/**
+ * Initialize the memcached stats plugin.
+ *
+ * @param[in] registry the drizzled::plugin::Registry singleton
+ * @return false on success; true on failure.
+ */
+static int init(plugin::Registry &registry)
+{
+  if (initMethods())
+  {
+    return true;
+  }
+
+  if (initColumns())
+  {
+    return true;
+  }
+
+  if (initMemcachedTables())
+  {
+    return true;
+  }
+
+  /* we are good to go */
+  registry.add(memcached_stats_table);
+
+  return false;
+}
+
+/**
+ * Clean up the memcached stats plugin.
+ *
+ * @param[in] registry the drizzled::plugin::Registry singleton
+ * @return false on success; true on failure
+ */
+static int deinit(plugin::Registry &registry)
+{
+  registry.remove(memcached_stats_table);
+
+  cleanupMethods();
+  cleanupColumns();
+  cleanupMemcachedTables();
+
+  return false;
+}
+
+static DRIZZLE_SYSVAR_STR(servers,
+                          sysvar_memcached_servers,
+                          PLUGIN_VAR_OPCMDARG,
+                          N_("List of memcached servers."),
+                          NULL, /* check func */
+                          NULL, /* update func */
+                          DEFAULT_SERVERS_STRING);
+
+static struct st_mysql_sys_var *system_variables[]=
+{
+  DRIZZLE_SYSVAR(servers),
+  NULL
+};
+
+drizzle_declare_plugin(memcached_stats)
+{
+  "memcached_stats",
+  "0.1",
+  "Padraig O'Sullivan",
+  N_("Memcached Stats as I_S tables"),
+  PLUGIN_LICENSE_GPL,
+  init,   /* Plugin Init      */
+  deinit, /* Plugin Deinit    */
+  NULL,   /* status variables */
+  system_variables, /* system variables */
+  NULL    /* config options   */
+}
+drizzle_declare_plugin_end;

=== added file 'plugin/memcached_stats/plugin.ac'
--- plugin/memcached_stats/plugin.ac	1970-01-01 00:00:00 +0000
+++ plugin/memcached_stats/plugin.ac	2009-09-29 19:34:58 +0000
@@ -0,0 +1,9 @@
+AC_LIB_HAVE_LINKFLAGS(memcached,,
+  [#include <libmemcached/memcached.h>],
+  [
+    memcached_st memc;
+    memcached_lib_version();
+  ])
+AS_IF([test "x$ac_cv_libmemcached" = "xno"],
+      AC_MSG_WARN([libmemcached not found: not building memcached engine.]))
+DRIZZLED_PLUGIN_DEP_LIBS="${DRIZZLED_PLUGIN_DEP_LIBS} ${LTLIBMEMCACHED}"

=== added file 'plugin/memcached_stats/plugin.ini'
--- plugin/memcached_stats/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/memcached_stats/plugin.ini	2009-09-29 19:34:58 +0000
@@ -0,0 +1,9 @@
+[plugin]
+name=memcached_stats
+title=Memcached Stats in I_S tables
+description=Some I_S tables that provide Memcached stats
+load_by_default=yes
+headers=stats_table.h
+sources=memcached_stats.cc stats_table.cc 
+build_conditional="${ac_cv_libmemcached}" = "yes"
+ldflags=${LTLIBMEMCACHED}

=== added file 'plugin/memcached_stats/stats_table.cc'
--- plugin/memcached_stats/stats_table.cc	1970-01-01 00:00:00 +0000
+++ plugin/memcached_stats/stats_table.cc	2009-09-29 19:34:58 +0000
@@ -0,0 +1,418 @@
+/*
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "stats_table.h"
+
+#include <libmemcached/memcached.h>
+
+#include <string>
+#include <vector>
+
+using namespace std;
+
+int MemcachedStatsISMethods::fillTable(Session *session,
+                                       TableList *tables,
+                                       COND *)
+{
+  const CHARSET_INFO * const scs= system_charset_info;
+  Table *table= tables->table;
+
+  table->restoreRecordAsDefault();
+
+  memcached_return rc;
+  memcached_st *serv= memcached_create(NULL);
+  memcached_server_st *tmp_serv=
+    memcached_servers_parse(servers_string.c_str());
+  memcached_server_push(serv, tmp_serv);
+  memcached_server_list_free(tmp_serv);
+  memcached_stat_st *stats= memcached_stat(serv, NULL, &rc);
+  memcached_server_st *servers= memcached_server_list(serv);
+
+  for (uint32_t i= 0; i < memcached_server_count(serv); i++)
+  {
+    char **list= memcached_stat_get_keys(serv, &stats[i], &rc);
+    char **ptr= NULL;
+
+    table->field[0]->store(memcached_server_name(serv, servers[i]),
+                           64,
+                           scs);
+    table->field[1]->store(memcached_server_port(serv, servers[i]));
+    uint32_t col= 2;
+    for (ptr= list; *ptr; ptr++)
+    {
+      char *value= memcached_stat_get_value(serv, &stats[i], *ptr, &rc);
+      table->field[col]->store(value,
+                               64,
+                               scs);
+      col++;
+      free(value);
+    }
+    free(list);
+  }
+
+  memcached_stat_free(serv, stats);
+  memcached_free(serv);
+
+  /* store the actual record now */
+  if (schema_table_store_record(session, table))
+  {
+    return 1;
+  }
+
+  return 0;
+}
+
+bool createMemcachedStatsColumns(vector<const ColumnInfo *> &cols)
+{
+  /*
+   * Create each column for the memcached stats table.
+   */
+  const ColumnInfo *name_col= new(std::nothrow) ColumnInfo("NAME",
+                                                           32,
+                                                           DRIZZLE_TYPE_VARCHAR,
+                                                           0,
+                                                           0,
+                                                           "Name",
+                                                           SKIP_OPEN_TABLE);
+  if (! name_col)
+  {
+    return true;
+  }
+
+  const ColumnInfo *port= new(std::nothrow) ColumnInfo("PORT_NUMBER",
+                                                       4,
+                                                       DRIZZLE_TYPE_LONGLONG,
+                                                       0,
+                                                       0, 
+                                                       "Port Number",
+                                                       SKIP_OPEN_TABLE);
+  if (! port)
+  {
+    return true;
+  }
+
+  const ColumnInfo *pid= new(std::nothrow) ColumnInfo("PROCESS_ID",
+                                                      4,
+                                                      DRIZZLE_TYPE_LONGLONG,
+                                                      0,
+                                                      0, 
+                                                      "Process ID",
+                                                      SKIP_OPEN_TABLE);
+  if (! pid)
+  {
+    return true;
+  }
+
+  const ColumnInfo *uptime= new(std::nothrow) ColumnInfo("UPTIME",
+                                                         4,
+                                                         DRIZZLE_TYPE_LONGLONG,
+                                                         0,
+                                                         0, 
+                                                         "Uptime",
+                                                         SKIP_OPEN_TABLE);
+  if (! uptime)
+  {
+    return true;
+  }
+
+  const ColumnInfo *time= new(std::nothrow) ColumnInfo("TIME",
+                                                       4,
+                                                       DRIZZLE_TYPE_LONGLONG,
+                                                       0,
+                                                       0, 
+                                                       "Time",
+                                                       SKIP_OPEN_TABLE);
+  if (! time)
+  {
+    return true;
+  }
+
+  const ColumnInfo *version= new(std::nothrow) ColumnInfo("VERSION",
+                                                          8,
+                                                          DRIZZLE_TYPE_VARCHAR,
+                                                          0,
+                                                          0,
+                                                          "Version",
+                                                          SKIP_OPEN_TABLE);
+  if (! version)
+  {
+    return true;
+  }
+
+  const ColumnInfo *ptr_size= new(std::nothrow) ColumnInfo("POINTER_SIZE",
+                                                           4,
+                                                           DRIZZLE_TYPE_LONGLONG,
+                                                           0,
+                                                           0, 
+                                                           "Pointer Size",
+                                                           SKIP_OPEN_TABLE);
+  if (! ptr_size)
+  {
+    return true;
+  }
+
+  const ColumnInfo *r_user= new(std::nothrow) ColumnInfo("RUSAGE_USER",
+                                                         4,
+                                                         DRIZZLE_TYPE_LONGLONG,
+                                                         0,
+                                                         0, 
+                                                         "rusage user",
+                                                         SKIP_OPEN_TABLE);
+  if (! r_user)
+  {
+    return true;
+  }
+
+  const ColumnInfo *r_sys= new(std::nothrow) ColumnInfo("RUSAGE_SYSTEM",
+                                                        4,
+                                                        DRIZZLE_TYPE_LONGLONG,
+                                                        0,
+                                                        0, 
+                                                        "rusage system",
+                                                        SKIP_OPEN_TABLE);
+  if (! r_sys)
+  {
+    return true;
+  }
+  const ColumnInfo *curr_items= new(std::nothrow) ColumnInfo("CURRENT_ITEMS",
+                                                             4,
+                                                             DRIZZLE_TYPE_LONGLONG,
+                                                             0,
+                                                             0, 
+                                                             "Current Items",
+                                                             SKIP_OPEN_TABLE);
+  if (! curr_items)
+  {
+    return true;
+  }
+
+  const ColumnInfo *total_items= new(std::nothrow) ColumnInfo("TOTAL_ITEMS",
+                                                              4,
+                                                              DRIZZLE_TYPE_LONGLONG,
+                                                              0,
+                                                              0,
+                                                              "Total Items",
+                                                              SKIP_OPEN_TABLE);
+  if (! total_items)
+  {
+    return true;
+  }
+
+  const ColumnInfo *bytes= new(std::nothrow) ColumnInfo("BYTES",
+                                                        4,
+                                                        DRIZZLE_TYPE_LONGLONG,
+                                                        0,
+                                                        0,
+                                                        "Bytes",
+                                                        SKIP_OPEN_TABLE);
+  if (! bytes)
+  {
+    return true;
+  }
+
+  const ColumnInfo *curr_cons= new(std::nothrow) ColumnInfo("CURRENT_CONNECTIONS",
+                                                            4,
+                                                            DRIZZLE_TYPE_LONGLONG,
+                                                            0,
+                                                            0,
+                                                            "Current Connections",
+                                                            SKIP_OPEN_TABLE);
+  if (! curr_cons)
+  {
+    return true;
+  }
+
+  const ColumnInfo *total_cons= new(std::nothrow) ColumnInfo("TOTAL_CONNECTIONS",
+                                                             4,
+                                                             DRIZZLE_TYPE_LONGLONG,
+                                                             0,
+                                                             0,
+                                                             "Total Connections",
+                                                             SKIP_OPEN_TABLE);
+  if (! total_cons)
+  {
+    return true;
+  }
+
+  const ColumnInfo *con_structs= new(std::nothrow) ColumnInfo("CONNECTION_STRUCTURES",
+                                                              4,
+                                                              DRIZZLE_TYPE_LONGLONG,
+                                                              0,
+                                                              0,
+                                                              "Connection Structures",
+                                                              SKIP_OPEN_TABLE);
+  if (! con_structs)
+  {
+    return true;
+  }
+
+  const ColumnInfo *cmd_gets= new(std::nothrow) ColumnInfo("GETS",
+                                                           4,
+                                                           DRIZZLE_TYPE_LONGLONG,
+                                                           0,
+                                                           0,
+                                                           "Gets",
+                                                           SKIP_OPEN_TABLE);
+  if (! cmd_gets)
+  {
+    return true;
+  }
+
+  const ColumnInfo *cmd_sets= new(std::nothrow) ColumnInfo("SETS",
+                                                           4,
+                                                           DRIZZLE_TYPE_LONGLONG,
+                                                           0,
+                                                           0,
+                                                           "Sets",
+                                                           SKIP_OPEN_TABLE);
+  if (! cmd_sets)
+  {
+    return true;
+  }
+
+  const ColumnInfo *hits= new(std::nothrow) ColumnInfo("HITS",
+                                                       4,
+                                                       DRIZZLE_TYPE_LONGLONG,
+                                                       0,
+                                                       0,
+                                                       "Hits",
+                                                       SKIP_OPEN_TABLE);
+  if (! hits)
+  {
+    return true;
+  }
+
+  const ColumnInfo *misses= new(std::nothrow) ColumnInfo("MISSES",
+                                                         4,
+                                                         DRIZZLE_TYPE_LONGLONG,
+                                                         0,
+                                                         0,
+                                                         "Misses",
+                                                         SKIP_OPEN_TABLE);
+  if (! misses)
+  {
+    return true;
+  }
+
+  const ColumnInfo *evicts= new(std::nothrow) ColumnInfo("EVICTIONS",
+                                                         4,
+                                                         DRIZZLE_TYPE_LONGLONG,
+                                                         0,
+                                                         0,
+                                                         "Evictions",
+                                                         SKIP_OPEN_TABLE);
+  if (! evicts)
+  {
+    return true;
+  }
+
+  const ColumnInfo *bytes_read= new(std::nothrow) ColumnInfo("BYTES_READ",
+                                                             4,
+                                                             DRIZZLE_TYPE_LONGLONG,
+                                                             0,
+                                                             0,
+                                                             "bytes read",
+                                                             SKIP_OPEN_TABLE);
+  if (! bytes_read)
+  {
+    return true;
+  }
+
+  const ColumnInfo *bytes_written= new(std::nothrow) ColumnInfo("BYTES_WRITTEN",
+                                                                4,
+                                                                DRIZZLE_TYPE_LONGLONG,
+                                                                0,
+                                                                0,
+                                                                "bytes written",
+                                                                SKIP_OPEN_TABLE);
+  if (! bytes_written)
+  {
+    return true;
+  }
+
+  const ColumnInfo *lim_max_bytes= new(std::nothrow) ColumnInfo("LIMIT_MAXBYTES",
+                                                                4,
+                                                                DRIZZLE_TYPE_LONGLONG,
+                                                                0,
+                                                                0,
+                                                                "limit maxbytes",
+                                                                SKIP_OPEN_TABLE);
+  if (! lim_max_bytes)
+  {
+    return true;
+  }
+
+  const ColumnInfo *threads= new(std::nothrow) ColumnInfo("THREADS",
+                                                          4,
+                                                          DRIZZLE_TYPE_LONGLONG,
+                                                          0,
+                                                          0,
+                                                          "Threads",
+                                                          SKIP_OPEN_TABLE);
+  if (! threads)
+  {
+    return true;
+  }
+
+  cols.push_back(name_col);
+  cols.push_back(port);
+  cols.push_back(pid);
+  cols.push_back(uptime);
+  cols.push_back(time);
+  cols.push_back(version);
+  cols.push_back(ptr_size);
+  cols.push_back(r_user);
+  cols.push_back(r_sys);
+  cols.push_back(curr_items);
+  cols.push_back(total_items);
+  cols.push_back(bytes);
+  cols.push_back(curr_cons);
+  cols.push_back(total_cons);
+  cols.push_back(con_structs);
+  cols.push_back(cmd_gets);
+  cols.push_back(cmd_sets);
+  cols.push_back(hits);
+  cols.push_back(misses);
+  cols.push_back(evicts);
+  cols.push_back(bytes_read);
+  cols.push_back(bytes_written);
+  cols.push_back(lim_max_bytes);
+  cols.push_back(threads);
+
+  return false;
+}
+
+class DeleteMemcachedCols
+{
+public:
+  template<typename T>
+  inline void operator()(const T *ptr) const
+  {
+    delete ptr;
+  }
+};
+
+void clearMemcachedColumns(vector<const ColumnInfo *> &cols)
+{
+  for_each(cols.begin(), cols.end(), DeleteMemcachedCols());
+  cols.clear();
+}

=== added file 'plugin/memcached_stats/stats_table.h'
--- plugin/memcached_stats/stats_table.h	1970-01-01 00:00:00 +0000
+++ plugin/memcached_stats/stats_table.h	2009-09-29 19:34:58 +0000
@@ -0,0 +1,51 @@
+/* 
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_MEMCACHED_STATS_STATS_TABLE_H
+#define PLUGIN_MEMCACHED_STATS_STATS_TABLE_H
+
+#include "drizzled/info_schema.h"
+
+#include <vector>
+
+class MemcachedStatsISMethods : public InfoSchemaMethods
+{
+public:
+  MemcachedStatsISMethods(const std::string &in_servers)
+    :
+      InfoSchemaMethods(),
+      servers_string(in_servers)
+  {}
+
+  virtual int fillTable(Session *session,
+                        TableList *tables,
+                        COND *cond);
+
+  void setServersString(const std::string &in_servers)
+  {
+    servers_string.assign(in_servers);
+  }
+
+private:
+  std::string servers_string;
+};
+
+bool createMemcachedStatsColumns(std::vector<const ColumnInfo *> &cols);
+
+void clearMemcachedColumns(std::vector<const ColumnInfo *> &cols);
+
+#endif /* PLUGIN_MEMCACHED_STATS_STATS_TABLE_H */

